# *****************************************************************
#
# Licensed Materials - Property of IBM 
#
# (C) Copyright IBM Corp. 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# ******************************************************************

from scipy import sparse
from _simsearch import libpywmd

import os
#import psutil

def _get_mpi_local_size():
    val = os.getenv('OMPI_COMM_WORLD_LOCAL_SIZE')
    if val is None:
        return 1
    else:
        return int(val)

def _compute_num_threads(verbose=False):

    affinity_count = None

    if hasattr(os, 'sched_getaffinity'):
        affinity_count = len(os.sched_getaffinity(0))
        if (verbose):
            print("os supports sched_getaffinity: affinity_count = " + str(affinity_count))

#    if affinity_count is None:
#        proc = psutil.Process()
#        if hasattr(proc, 'cpu_affinity'):
#            affinity_count = len(proc.cpu_affinity())
#            if (verbose):
#                print("psutil supports cpu_affinity: affinity_count = " + str(affinity_count))

    cpu_count = None

    if hasattr(os, 'cpu_count'):
        cpu_count = os.cpu_count()
        if (verbose):
            print("os supports cpu_count: cpu_count = " + str(cpu_count))

    if cpu_count is None:
        cpu_count = affinity_count
        if (verbose):
            print("cpu_count = " + str(cpu_count))
    
    process_count = _get_mpi_local_size()  
    if (verbose):
        print("process_count = " + str(process_count))

    if cpu_count is not None:
        num_threads = int(cpu_count / process_count)
        if affinity_count is not None:
            num_threads = min(num_threads, affinity_count)
        num_threads = max(1, num_threads) 
    else:
        num_threads = 1

    if (verbose):
        print("num_threads = " + str(num_threads))

    return num_threads

#######################################
# COMPUTE WORD MOVERS DISTANCE
def word_movers_distance(W, X, Y=None, use_gpu=True, optimization_level=1, num_cpu_threads=None):
    """Computes word movers distance between samples in X and Y.

    The details of the approximation algorithm can be found here:
    "http://export.arxiv.org/abs/1812.02091".

    Parameters
    ----------
    W : ndarray, shape: (n_features, n_dimensions)
        Embedding vectors

    X : array-like, sparse-matrix, shape (n_samples_x, n_features)
        Input dataset. When MPI is used, it is preferred to have X of larger size than Y when X and Y are of 
        unequal sizes as the MPI distribution of data is performed for X only.

    Y : array-like, sparse-matrix, shape (n_samples_y, n_features), optional
        Input dataset. If ``None``, the output will be the pairwise similarities between all samples in ``X``.

    optimization_level : integer value between 0 and 4, using a high value increases both accuracy and runtime.

    num_cpu_threads : Maximum number of CPU threads used by each process. 

    Returns
    -------
    D : array-like, shape (n_samples_x, n_samples_y)
        Word Movers Distance
    """

    if num_cpu_threads is None:
        num_cpu_threads = _compute_num_threads()

    if Y is None:
        Y=X

    if (use_gpu==True):
        D = libpywmd.wmd_wrapper(W, X.nnz, X.indptr, X.indices, X.data, Y.nnz, Y.indptr, Y.indices, Y.data, 1, optimization_level, num_cpu_threads)
    else:
        D = libpywmd.wmd_wrapper(W, X.nnz, X.indptr, X.indices, X.data, Y.nnz, Y.indptr, Y.indices, Y.data, 0, optimization_level, num_cpu_threads)

    return(D)

#######################################
